@@ -47,6 +47,12 @@ serf_bucket_t *serf_bucket_request_create(
     serf_bucket_t *body,
     serf_bucket_alloc_t *allocator);
 
+/* Send a Content-Length header with @a len. The @a body bucket should
+   contain precisely that much data.  */
+void serf_bucket_request_set_CL(
+    serf_bucket_t *bucket,
+    apr_int64_t len);
+
 serf_bucket_t *serf_bucket_request_get_headers(
     serf_bucket_t *request);
 
@@ -486,6 +492,13 @@ typedef apr_status_t (*serf_ssl_need_server_cert_t)(
     int failures,
     const serf_ssl_certificate_t *cert);
 
+typedef apr_status_t (*serf_ssl_server_cert_chain_cb_t)(
+    void *data,
+    int failures,
+    int error_depth,
+    const serf_ssl_certificate_t * const * certs,
+    apr_size_t certs_len);
+
 void serf_ssl_client_cert_provider_set(
     serf_ssl_context_t *context,
     serf_ssl_need_client_cert_t callback,
@@ -507,6 +520,16 @@ void serf_ssl_server_cert_callback_set(
     serf_ssl_need_server_cert_t callback,
     void *data);
 
+/**
+ * Set callbacks to override the default SSL server certificate validation 
+ * algorithm for the current certificate or the entire certificate chain. 
+ */
+void serf_ssl_server_cert_chain_callback_set(
+    serf_ssl_context_t *context,
+    serf_ssl_need_server_cert_t cert_callback,
+    serf_ssl_server_cert_chain_cb_t cert_chain_callback,
+    void *data);
+
 /**
  * Use the default root CA certificates as included with the OpenSSL library.
  */
