@@ -23,6 +23,7 @@
 #include "serf.h"
 
 #include "test_serf.h"
+#include "server/test_server.h"
 
 typedef struct {
     serf_response_acceptor_t acceptor;
@@ -39,9 +40,6 @@ typedef struct {
     const char *path;
     int done;
 
-    const char *server_root;
-    int use_proxy;
-
     test_baton_t *tb;
 } handler_baton_t;
 
@@ -126,6 +124,10 @@ static apr_status_t handle_response(serf_request_t *request,
             return APR_EOF;
         }
 
+        if (APR_STATUS_IS_EAGAIN(status)) {
+            return status;
+        }
+
     }
 
     return APR_SUCCESS;
@@ -141,44 +143,26 @@ static void test_serf_connection_request_create(CuTest *tc)
     apr_pool_t *iter_pool;
     apr_array_header_t *accepted_requests, *handled_requests, *sent_requests;
     int i;
-    test_server_action_t action_list[] = {
-        {SERVER_RECV,
-        "GET / HTTP/1.1" CRLF
-        "Transfer-Encoding: chunked" CRLF
-        CRLF
-        "1" CRLF
-        "1" CRLF
-        "0" CRLF
-        CRLF
-        "GET / HTTP/1.1" CRLF
-        "Transfer-Encoding: chunked" CRLF
-        "" CRLF
-        "1" CRLF
-        "2" CRLF
-        "0" CRLF
-        CRLF
-        },
-        {SERVER_SEND,
-        "HTTP/1.1 200 OK" CRLF
-        "Transfer-Encoding: chunked" CRLF
-        CRLF
-        "0" CRLF
-        CRLF
-        "HTTP/1.1 200 OK" CRLF
-        "Transfer-Encoding: chunked" CRLF
-        CRLF
-        "0" CRLF
-        CRLF
-        }
+    test_server_message_t message_list[] = {
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "2")},
     };
 
+    test_server_action_t action_list[] = {
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+    };
+    apr_pool_t *test_pool = test_setup();
+
     accepted_requests = apr_array_make(test_pool, 2, sizeof(int));
     sent_requests = apr_array_make(test_pool, 2, sizeof(int));
     handled_requests = apr_array_make(test_pool, 2, sizeof(int));
 
     /* Set up a test context with a server */
-    status = test_server_create(&tb, action_list, 2, 0, NULL, NULL, NULL,
-                                test_pool);
+    status = test_server_setup(&tb,
+                               message_list, 2,
+                               action_list, 2, 0, NULL,
+                               test_pool);
     CuAssertIntEquals(tc, APR_SUCCESS, status);
 
     handler_ctx.method = "GET";
@@ -192,8 +176,6 @@ static void test_serf_connection_request_create(CuTest *tc)
     handler_ctx.accepted_requests = accepted_requests;
     handler_ctx.sent_requests = sent_requests;
     handler_ctx.handled_requests = handled_requests;
-    handler_ctx.use_proxy = FALSE;
-    handler_ctx.server_root = NULL;
 
     request1 = serf_connection_request_create(tb->connection,
                                               setup_request,
@@ -208,11 +190,11 @@ static void test_serf_connection_request_create(CuTest *tc)
 
     apr_pool_create(&iter_pool, test_pool);
 
-    while (!handler_ctx.done && !handler2_ctx.done)
+    while (!handler_ctx.done || !handler2_ctx.done)
     {
         apr_pool_clear(iter_pool);
 
-        status = test_server_run(tb, 0, iter_pool);
+        status = test_server_run(tb->serv_ctx, 0, iter_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
@@ -221,9 +203,6 @@ static void test_serf_connection_request_create(CuTest *tc)
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
-
-        /* Debugging purposes only! */
-        serf_debug__closed_conn(tb->bkt_alloc);
     }
     apr_pool_destroy(iter_pool);
 
@@ -244,7 +223,8 @@ static void test_serf_connection_request_create(CuTest *tc)
         CuAssertIntEquals(tc, i + 1, req_nr);
     }
 
-    test_server_destroy(tb, test_pool);
+    test_server_teardown(tb, test_pool);
+    test_teardown(test_pool);
 }
 
 /* Validate that priority requests are sent and completed before normal
@@ -258,26 +238,30 @@ static void test_serf_connection_priority_request_create(CuTest *tc)
     apr_pool_t *iter_pool;
     apr_array_header_t *accepted_requests, *handled_requests, *sent_requests;
     int i;
-    test_server_action_t action_list[] = {
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-         CHUNCKED_REQUEST(1, "2")
-         CHUNCKED_REQUEST(1, "3")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-        }
+
+    test_server_message_t message_list[] = {
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "2")},
+        {CHUNKED_REQUEST(1, "3")},
     };
 
+    test_server_action_t action_list[] = {
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+    };
+
+    apr_pool_t *test_pool = test_setup();
+
     accepted_requests = apr_array_make(test_pool, 3, sizeof(int));
     sent_requests = apr_array_make(test_pool, 3, sizeof(int));
     handled_requests = apr_array_make(test_pool, 3, sizeof(int));
 
     /* Set up a test context with a server */
-    status = test_server_create(&tb, action_list, 2, 0, NULL, NULL, NULL,
-                                test_pool);
+    status = test_server_setup(&tb,
+                               message_list, 3,
+                               action_list, 3, 0, NULL,
+                               test_pool);
     CuAssertIntEquals(tc, APR_SUCCESS, status);
 
     handler_ctx.method = "GET";
@@ -291,8 +275,6 @@ static void test_serf_connection_priority_request_create(CuTest *tc)
     handler_ctx.accepted_requests = accepted_requests;
     handler_ctx.sent_requests = sent_requests;
     handler_ctx.handled_requests = handled_requests;
-    handler_ctx.use_proxy = FALSE;
-    handler_ctx.server_root = NULL;
 
     request1 = serf_connection_request_create(tb->connection,
                                               setup_request,
@@ -313,11 +295,11 @@ static void test_serf_connection_priority_request_create(CuTest *tc)
 
     apr_pool_create(&iter_pool, test_pool);
 
-    while (!handler_ctx.done && !handler2_ctx.done && !handler3_ctx.done)
+    while (!handler_ctx.done || !handler2_ctx.done || !handler3_ctx.done)
     {
         apr_pool_clear(iter_pool);
 
-        status = test_server_run(tb, 0, iter_pool);
+        status = test_server_run(tb->serv_ctx, 0, iter_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
@@ -349,7 +331,8 @@ static void test_serf_connection_priority_request_create(CuTest *tc)
         CuAssertIntEquals(tc, i + 1, req_nr);
     }
 
-    test_server_destroy(tb, test_pool);
+    test_server_teardown(tb, test_pool);
+    test_teardown(test_pool);
 }
 
 /* Test that serf correctly handles the 'Connection:close' header when the
@@ -363,23 +346,24 @@ static void test_serf_closed_connection(CuTest *tc)
     handler_baton_t handler_ctx[NUM_REQUESTS];
     int done = FALSE, i;
 
+    test_server_message_t message_list[] = {
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "2")},
+        {CHUNKED_REQUEST(1, "3")},
+        {CHUNKED_REQUEST(1, "4")},
+        {CHUNKED_REQUEST(1, "5")},
+        {CHUNKED_REQUEST(1, "6")},
+        {CHUNKED_REQUEST(1, "7")},
+        {CHUNKED_REQUEST(1, "8")},
+        {CHUNKED_REQUEST(1, "9")},
+        {CHUNKED_REQUEST(2, "10")}
+        };
+
     test_server_action_t action_list[] = {
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-         CHUNCKED_REQUEST(1, "2")
-         CHUNCKED_REQUEST(1, "3")
-         CHUNCKED_REQUEST(1, "4")
-         CHUNCKED_REQUEST(1, "5")
-         CHUNCKED_REQUEST(1, "6")
-         CHUNCKED_REQUEST(1, "7")
-         CHUNCKED_REQUEST(1, "8")
-         CHUNCKED_REQUEST(1, "9")
-         CHUNCKED_REQUEST(2, "10")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND,
          "HTTP/1.1 200 OK" CRLF
          "Transfer-Encoding: chunked" CRLF
          "Connection: close" CRLF
@@ -387,11 +371,11 @@ static void test_serf_closed_connection(CuTest *tc)
          "0" CRLF
          CRLF
         },
-        {SERVER_KILL_CONNECTION},
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
+        {SERVER_IGNORE_AND_KILL_CONNECTION},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND,
          "HTTP/1.1 200 OK" CRLF
          "Transfer-Encoding: chunked" CRLF
          "Connection: close" CRLF
@@ -399,20 +383,24 @@ static void test_serf_closed_connection(CuTest *tc)
          "0" CRLF
          CRLF
         },
-        {SERVER_KILL_CONNECTION},
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-        },
+        {SERVER_IGNORE_AND_KILL_CONNECTION},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
     };
 
+    apr_pool_t *test_pool = test_setup();
+
     accepted_requests = apr_array_make(test_pool, NUM_REQUESTS, sizeof(int));
     sent_requests = apr_array_make(test_pool, NUM_REQUESTS, sizeof(int));
     handled_requests = apr_array_make(test_pool, NUM_REQUESTS, sizeof(int));
 
     /* Set up a test context with a server. */
-    status = test_server_create(&tb, action_list, 6, 0, NULL, NULL, NULL,
-                                test_pool);
+    status = test_server_setup(&tb,
+                               message_list, 10,
+                               action_list, 12,
+                               0,
+                               NULL,
+                               test_pool);
     CuAssertIntEquals(tc, APR_SUCCESS, status);
 
     for (i = 0 ; i < NUM_REQUESTS ; i++) {
@@ -429,8 +417,6 @@ static void test_serf_closed_connection(CuTest *tc)
         handler_ctx[i].sent_requests = sent_requests;
         handler_ctx[i].handled_requests = handled_requests;
         handler_ctx[i].tb = tb;
-        handler_ctx[i].use_proxy = FALSE;
-        handler_ctx[i].server_root = NULL;
 
         serf_connection_request_create(tb->connection,
                                        setup_request,
@@ -438,7 +424,7 @@ static void test_serf_closed_connection(CuTest *tc)
     }
 
     while (1) {
-        status = test_server_run(tb, 0, test_pool);
+        status = test_server_run(tb->serv_ctx, 0, test_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
@@ -467,7 +453,8 @@ static void test_serf_closed_connection(CuTest *tc)
     CuAssertIntEquals(tc, NUM_REQUESTS, handled_requests->nelts);
 
     /* Cleanup */
-    test_server_destroy(tb, test_pool);
+    test_server_teardown(tb, test_pool);
+    test_teardown(test_pool);
 }
 #undef NUM_REQUESTS
 
@@ -475,7 +462,7 @@ static void test_serf_closed_connection(CuTest *tc)
    directly. */
 static void test_serf_setup_proxy(CuTest *tc)
 {
-    test_baton_t *tb_server, *tb_proxy;
+    test_baton_t *tb;
     serf_request_t *request;
     handler_baton_t handler_ctx;
     apr_status_t status;
@@ -483,44 +470,38 @@ static void test_serf_setup_proxy(CuTest *tc)
     apr_array_header_t *accepted_requests, *handled_requests, *sent_requests;
     int i;
     int numrequests = 1;
-    apr_sockaddr_t *proxy_address;
-    test_server_action_t *action_list_server = NULL;
-    test_server_action_t action_list_proxy[] = {
-        {SERVER_RECV,
-         "GET http://localhost:" SERV_PORT_STR " HTTP/1.1" CRLF\
+
+    test_server_message_t message_list[] = {
+        {"GET http://localhost:" SERV_PORT_STR " HTTP/1.1" CRLF\
+         "Host: localhost:" SERV_PORT_STR CRLF\
          "Transfer-Encoding: chunked" CRLF\
-         "Host: localhost" CRLF\
          CRLF\
          "1" CRLF\
          "1" CRLF\
          "0" CRLF\
-         CRLF
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        }
+         CRLF}
     };
 
+    test_server_action_t action_list_proxy[] = {
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+    };
+
+    apr_pool_t *test_pool = test_setup();
+
     accepted_requests = apr_array_make(test_pool, numrequests, sizeof(int));
     sent_requests = apr_array_make(test_pool, numrequests, sizeof(int));
     handled_requests = apr_array_make(test_pool, numrequests, sizeof(int));
 
-    /* Set up a test context with a server */
-    status = test_server_create(&tb_server, action_list_server, 2, 0,
-                                "http://localhost:" SERV_PORT_STR, NULL,
-                                NULL, test_pool);
-    CuAssertIntEquals(tc, APR_SUCCESS, status);
-
-    /* Set up another test context for the proxy server */
-    status = apr_sockaddr_info_get(&proxy_address,
-                                   "localhost", APR_INET, 21212, 0,
-                                   test_pool);
-    CuAssertIntEquals(tc, APR_SUCCESS, status);
-
-    status = test_server_create(&tb_proxy, action_list_proxy, 2, 0,
-                                NULL, proxy_address, NULL,
-                                test_pool);
-
+    /* Set up a test context with a server, no messages expected. */
+    status = test_server_proxy_setup(&tb,
+                                     /* server messages and actions */
+                                     NULL, 0,
+                                     NULL, 0,
+                                     /* server messages and actions */
+                                     message_list, 1,
+                                     action_list_proxy, 1,
+                                     0,
+                                     NULL, test_pool);
     CuAssertIntEquals(tc, APR_SUCCESS, status);
 
     handler_ctx.method = "GET";
@@ -534,13 +515,8 @@ static void test_serf_setup_proxy(CuTest *tc)
     handler_ctx.accepted_requests = accepted_requests;
     handler_ctx.sent_requests = sent_requests;
     handler_ctx.handled_requests = handled_requests;
-    handler_ctx.use_proxy = TRUE;
-    handler_ctx.server_root = "http://localhost:" SERV_PORT_STR;
 
-    /* Configure serf to use the proxy server */
-    serf_config_proxy(tb_server->context, proxy_address);
-
-    request = serf_connection_request_create(tb_server->connection,
+    request = serf_connection_request_create(tb->connection,
                                              setup_request,
                                              &handler_ctx);
 
@@ -550,23 +526,23 @@ static void test_serf_setup_proxy(CuTest *tc)
     {
         apr_pool_clear(iter_pool);
 
-        status = test_server_run(tb_server, 0, iter_pool);
+        status = test_server_run(tb->serv_ctx, 0, iter_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
 
-        status = test_server_run(tb_proxy, 0, iter_pool);
+        status = test_server_run(tb->proxy_ctx, 0, iter_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
 
-        status = serf_context_run(tb_server->context, 0, iter_pool);
+        status = serf_context_run(tb->context, 0, iter_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
 
         /* Debugging purposes only! */
-        serf_debug__closed_conn(tb_server->bkt_alloc);
+        serf_debug__closed_conn(tb->bkt_alloc);
     }
     apr_pool_destroy(iter_pool);
 
@@ -587,7 +563,8 @@ static void test_serf_setup_proxy(CuTest *tc)
         CuAssertIntEquals(tc, i + 1, req_nr);
     }
 
-    test_server_destroy(tb_server, test_pool);
+    test_server_teardown(tb, test_pool);
+    test_teardown(test_pool);
 }
 
 /*****************************************************************************
@@ -645,58 +622,37 @@ static void test_keepalive_limit_one_by_one(CuTest *tc)
     handler_baton_t handler_ctx[SEND_REQUESTS];
     int done = FALSE, i;
 
-    test_server_action_t action_list[] = {
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "2")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "3")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "4")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "5")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
+    test_server_message_t message_list[] = {
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "2")},
+        {CHUNKED_REQUEST(1, "3")},
+        {CHUNKED_REQUEST(1, "4")},
+        {CHUNKED_REQUEST(1, "5")},
     };
 
+    test_server_action_t action_list[] = {
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+    };
+
+    apr_pool_t *test_pool = test_setup();
+
     accepted_requests = apr_array_make(test_pool, RCVD_REQUESTS, sizeof(int));
     sent_requests = apr_array_make(test_pool, RCVD_REQUESTS, sizeof(int));
     handled_requests = apr_array_make(test_pool, RCVD_REQUESTS, sizeof(int));
 
     /* Set up a test context with a server. */
-    status = test_server_create(&tb, action_list, 14, 0, NULL, NULL, NULL,
-                                test_pool);
+    status = test_server_setup(&tb,
+                               message_list, 7,
+                               action_list, 7, 0, NULL,
+                               test_pool);
     CuAssertIntEquals(tc, APR_SUCCESS, status);
 
     for (i = 0 ; i < SEND_REQUESTS ; i++) {
@@ -713,8 +669,6 @@ static void test_keepalive_limit_one_by_one(CuTest *tc)
         handler_ctx[i].sent_requests = sent_requests;
         handler_ctx[i].handled_requests = handled_requests;
         handler_ctx[i].tb = tb;
-        handler_ctx[i].use_proxy = FALSE;
-        handler_ctx[i].server_root = NULL;
 
         serf_connection_request_create(tb->connection,
                                        setup_request,
@@ -723,7 +677,7 @@ static void test_keepalive_limit_one_by_one(CuTest *tc)
     }
 
     while (1) {
-        status = test_server_run(tb, 0, test_pool);
+        status = test_server_run(tb->serv_ctx, 0, test_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
@@ -752,7 +706,8 @@ static void test_keepalive_limit_one_by_one(CuTest *tc)
     CuAssertIntEquals(tc, RCVD_REQUESTS, handled_requests->nelts);
 
     /* Cleanup */
-    test_server_destroy(tb, test_pool);
+    test_server_teardown(tb, test_pool);
+    test_teardown(test_pool);
 }
 #undef SEND_REQUESTS
 #undef RCVD_REQUESTS
@@ -804,6 +759,10 @@ handle_response_keepalive_limit_burst(serf_request_t *request,
             }
             return APR_EOF;
         }
+
+        if (APR_STATUS_IS_EAGAIN(status)) {
+            return status;
+        }
     }
 
     return APR_SUCCESS;
@@ -817,46 +776,37 @@ static void test_keepalive_limit_one_by_one_and_burst(CuTest *tc)
     handler_baton_t handler_ctx[SEND_REQUESTS];
     int done = FALSE, i;
 
-    test_server_action_t action_list[] = {
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-        },
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "2")
-         CHUNCKED_REQUEST(1, "3")
-         CHUNCKED_REQUEST(1, "4")
-         CHUNCKED_REQUEST(1, "5")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-        },
+    test_server_message_t message_list[] = {
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "2")},
+        {CHUNKED_REQUEST(1, "3")},
+        {CHUNKED_REQUEST(1, "4")},
+        {CHUNKED_REQUEST(1, "5")},
     };
 
+    test_server_action_t action_list[] = {
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+    };
+
+    apr_pool_t *test_pool = test_setup();
+
     accepted_requests = apr_array_make(test_pool, RCVD_REQUESTS, sizeof(int));
     sent_requests = apr_array_make(test_pool, RCVD_REQUESTS, sizeof(int));
     handled_requests = apr_array_make(test_pool, RCVD_REQUESTS, sizeof(int));
 
     /* Set up a test context with a server. */
-    status = test_server_create(&tb, action_list, 8, 0, NULL, NULL, NULL, 
-                                test_pool);
+    status = test_server_setup(&tb,
+                               message_list, 7,
+                               action_list, 7, 0, NULL,
+                               test_pool);
     CuAssertIntEquals(tc, APR_SUCCESS, status);
 
     for (i = 0 ; i < SEND_REQUESTS ; i++) {
@@ -873,8 +823,6 @@ static void test_keepalive_limit_one_by_one_and_burst(CuTest *tc)
         handler_ctx[i].sent_requests = sent_requests;
         handler_ctx[i].handled_requests = handled_requests;
         handler_ctx[i].tb = tb;
-        handler_ctx[i].use_proxy = FALSE;
-        handler_ctx[i].server_root = NULL;
 
         serf_connection_request_create(tb->connection,
                                        setup_request,
@@ -883,7 +831,7 @@ static void test_keepalive_limit_one_by_one_and_burst(CuTest *tc)
     }
 
     while (1) {
-        status = test_server_run(tb, 0, test_pool);
+        status = test_server_run(tb->serv_ctx, 0, test_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
@@ -912,7 +860,8 @@ static void test_keepalive_limit_one_by_one_and_burst(CuTest *tc)
     CuAssertIntEquals(tc, RCVD_REQUESTS, handled_requests->nelts);
 
     /* Cleanup */
-    test_server_destroy(tb, test_pool);
+    test_server_teardown(tb, test_pool);
+    test_teardown(test_pool);
 }
 #undef SEND_REQUESTS
 #undef RCVD_REQUESTS
@@ -933,13 +882,15 @@ progress_cb(void *progress_baton, apr_off_t read, apr_off_t written)
     pb->written = written;
 }
 
-static serf_bucket_t* progress_conn_setup(apr_socket_t *skt,
+static apr_status_t progress_conn_setup(apr_socket_t *skt,
+                                          serf_bucket_t **input_bkt,
+                                          serf_bucket_t **output_bkt,
                                           void *setup_baton,
                                           apr_pool_t *pool)
 {
     test_baton_t *tb = setup_baton;
-
-    return serf_context_bucket_socket_create(tb->context, skt, tb->bkt_alloc);
+    *input_bkt = serf_context_bucket_socket_create(tb->context, skt, tb->bkt_alloc);
+    return APR_SUCCESS;
 }
 
 static void test_serf_progress_callback(CuTest *tc)
@@ -951,30 +902,33 @@ static void test_serf_progress_callback(CuTest *tc)
     int done = FALSE, i;
     progress_baton_t *pb;
 
-    test_server_action_t action_list[] = {
-        {SERVER_RECV,
-         CHUNCKED_REQUEST(1, "1")
-         CHUNCKED_REQUEST(1, "2")
-         CHUNCKED_REQUEST(1, "3")
-         CHUNCKED_REQUEST(1, "4")
-         CHUNCKED_REQUEST(1, "5")
-        },
-        {SERVER_SEND,
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_RESPONSE(1, "2")
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-         CHUNKED_EMPTY_RESPONSE
-        },
+    test_server_message_t message_list[] = {
+        {CHUNKED_REQUEST(1, "1")},
+        {CHUNKED_REQUEST(1, "2")},
+        {CHUNKED_REQUEST(1, "3")},
+        {CHUNKED_REQUEST(1, "4")},
+        {CHUNKED_REQUEST(1, "5")},
     };
 
+    test_server_action_t action_list[] = {
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_RESPONSE(1, "2")},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+        {SERVER_RESPOND, CHUNKED_EMPTY_RESPONSE},
+    };
+
+    apr_pool_t *test_pool = test_setup();
+
     accepted_requests = apr_array_make(test_pool, NUM_REQUESTS, sizeof(int));
     sent_requests = apr_array_make(test_pool, NUM_REQUESTS, sizeof(int));
     handled_requests = apr_array_make(test_pool, NUM_REQUESTS, sizeof(int));
 
     /* Set up a test context with a server. */
-    status = test_server_create(&tb, action_list, 2, 0, NULL, NULL, 
-                                progress_conn_setup, test_pool);
+    status = test_server_setup(&tb,
+                               message_list, 5,
+                               action_list, 5, 0,
+                               progress_conn_setup, test_pool);
     CuAssertIntEquals(tc, APR_SUCCESS, status);
 
     /* Set up the progress callback. */
@@ -996,8 +950,6 @@ static void test_serf_progress_callback(CuTest *tc)
         handler_ctx[i].sent_requests = sent_requests;
         handler_ctx[i].handled_requests = handled_requests;
         handler_ctx[i].tb = tb;
-        handler_ctx[i].use_proxy = FALSE;
-        handler_ctx[i].server_root = NULL;
 
         serf_connection_request_create(tb->connection,
                                        setup_request,
@@ -1005,7 +957,7 @@ static void test_serf_progress_callback(CuTest *tc)
     }
 
     while (1) {
-        status = test_server_run(tb, 0, test_pool);
+        status = test_server_run(tb->serv_ctx, 0, test_pool);
         if (APR_STATUS_IS_TIMEUP(status))
             status = APR_SUCCESS;
         CuAssertIntEquals(tc, APR_SUCCESS, status);
@@ -1038,7 +990,8 @@ static void test_serf_progress_callback(CuTest *tc)
     CuAssertTrue(tc, pb->read > 0);
 
     /* Cleanup */
-    test_server_destroy(tb, test_pool);
+    test_server_teardown(tb, test_pool);
+    test_teardown(test_pool);
 }
 #undef NUM_REQUESTS
 
